#!/usr/bin/perl
# make a logo-ish gnu-plottable representation of a score matrix

if( $#ARGV != 0 ){
	print "Usage: .model\n";
	print "    output is .logo\n";
	exit;
}

@parts = split('\.',$ARGV[0]);
$name = $parts[0];
for($i=1; $i< $#parts; $i++){
	$name = sprintf("%s.%s", $name, $parts[$i]);
}

#
# read in matrix
#
open(IN,"$ARGV[0]");
@re = ();
@mat = ();
@inds = ();

# burn 2 lines
$line = <IN>;
$line = <IN>;

while($line = <IN>){
	chomp($line);
	@parts = split(' ',$line);
	if( $parts[0] !~ /NA/ ){
		($ind, $reval) = split(';', $parts[0]);
		($ind1, $ind2) = split(',',$ind);
		push(@re, $reval);
		@tmp = ();
		for($i=1; $i<= $#parts; $i++){
			if( $parts[$i] ne "NA" ){push(@tmp, $parts[$i]);}
			else{push(@tmp, 0);}
		}
		push(@mat, [@tmp] );
		push(@inds, [($ind1, $ind2)]);
	}
}
close(IN);

#
# set all negative values to 0
#
for($i=0; $i<= $#mat; $i++){
	for($j=0; $j< $#parts; $j++){
		if( $mat[$i][$j]< 0 ){$mat[$i][$j]=0;}
	}
}

#
# rescale matrix so it's between 0 and 1, then multiply by re
#
for($i=0; $i<= $#mat; $i++){
	$min = 0;
	$max = 0;
	$sum = 0;
	for($j=0; $j< $#parts; $j++){
		if( $min > $mat[$i][$j] ){ $min = $mat[$i][$j]; }
		if( $max < $mat[$i][$j] ){ $max = $mat[$i][$j]; }
		$sum+= $mat[$i][$j];
	}
	for($j=0; $j< $#parts; $j++){
#		$mat[$i][$j] = $re[$i]*(($mat[$i][$j]-$min)/($max-$min));
		$mat[$i][$j] = $re[$i]*$mat[$i][$j]/$sum;
	}
}
	
#
# find max height row scaling
#
$remax = 0;
$min = 0;
$max = 0;
for($k=0; $k<= $#mat; $k++){
	@nvals = ($mat[$k][0], $mat[$k][1], $mat[$k][2]);
	@zvals = ($mat[$k][3], $mat[$k][4], $mat[$k][5]);
	@pvals = ($mat[$k][6], $mat[$k][7], $mat[$k][8]);
	$nht = 0;
	$zht = 0;
	$pht = 0;
	for($l=0; $l< 3; $l++){
		$nht+= $nvals[$l];
		$zht+= $zvals[$l];
		$pht+= $pvals[$l];
	}
	if( $min< $nht+0.5*$zht){$min = $nht+0.5*$zht;}
	if( $max< $pht+0.5*$zht){$max = $pht+0.5*$zht;}
}
$remax = $min+$max+0.1;

#
# find new index pairs
#
$ind1 = 1;
$ind2 = 1;
@newinds = ();
for($i=0; $i<= $#inds; $i++){
	if( $i>0 ){
		if( $inds[$i][1] != $inds[$i-1][1] ){$ind2++;}
		if( $inds[$i][0] != $inds[$i-1][0] ){
			$ind1++;
			$ind2 = $ind1;
		}
	}
	push( @newinds, [($ind1, $ind2)] );
}
for($i=0; $i<= $#inds; $i++){
	$tmp0 = $inds[$i][0];
	$tmp1 = $inds[$i][1];
	$inds[$i][0] = $newinds[$i][0];
	$inds[$i][1] = $newinds[$i][1];
	$newinds[$i][0] = $tmp0;
	$newinds[$i][1] = $tmp1;
}


#
# print result to plot in pm3d
#
open(OUT,">$name.logo");

$w = 0.9;
for($k=0; $k<= $#mat; $k++){
	if( $inds[$k][0] == $inds[$k][1] ){
		@nvals = ($mat[$k][0], $mat[$k][1], $mat[$k][2]);
		@zvals = ($mat[$k][3], $mat[$k][4], $mat[$k][5]);
		@pvals = ($mat[$k][6], $mat[$k][7], $mat[$k][8]);
		$nht = 0;
		$zht = 0;
		$pht = 0;
		for($l=0; $l< 3; $l++){
			$nht+= $nvals[$l];
			$zht+= $zvals[$l];
			$pht+= $pvals[$l];
		}
		@hts = ($nht, $zht, $pht);
		$min = -(0.5*$zht+$nht);
		
		#
		# print first column of zeros
		#
		$ht = $min;
		for($l=0; $l< 3; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1], $remax, 0);
			printf OUT ("%f %f %f\n", $inds[$k][1], $remax, 0);
			$ht+=$hts[$l];
		}
		printf OUT ("\n");

		#
		# print first column of boxes
		#
		$ht = $min;
		for($l=0; $l< 3; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1], $remax + $ht, 2*($l-1));
			printf OUT ("%f %f %f\n", $inds[$k][1], $remax + $ht+$hts[$l], 2*($l-1));
			$ht+=$hts[$l];
		}
		printf OUT ("\n");
		$ht = $min;
		for($l=0; $l< 3; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w/2, $remax + $ht, 2*($l-1));
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w/2, $remax + $ht+$hts[$l], 2*($l-1));
			$ht+=$hts[$l];
		}
		printf OUT ("\n");

		#
		# print second column of boxes
		#
		$ht = $min;
		for($l=0; $l< 9; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w/2, $remax + $ht, 2*($l%3-1));
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w/2, $remax + $ht+$mat[$k][$l], 2*($l%3-1));
			$ht+=$mat[$k][$l];
		}
		printf OUT ("\n");
		$ht = $min;
		for($l=0; $l< 9; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w, $remax + $ht, 2*($l%3-1));
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w, $remax + $ht+$mat[$k][$l], 2*($l%3-1));
			$ht+=$mat[$k][$l];
		}
		printf OUT ("\n");
		$ht = $min;
		for($l=0; $l< 9; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w, $remax, 0);
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w, $remax, 0);
			$ht+=$mat[$k][$l];
		}
		printf OUT ("\n");
		$ht = $min;
		for($l=0; $l< 9; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1]+1, $remax, 0);
			printf OUT ("%f %f %f\n", $inds[$k][1]+1, $remax, 0);
			$ht+=$mat[$k][$l];
		}
		printf OUT ("\n");
	}
}

for($k=0; $k<= $#mat; $k++){
	if( $inds[$k][0] != $inds[$k][1] ){
		@nvals = ($mat[$k][0], $mat[$k][1], $mat[$k][2]);
		@zvals = ($mat[$k][3], $mat[$k][4], $mat[$k][5]);
		@pvals = ($mat[$k][6], $mat[$k][7], $mat[$k][8]);
		$nht = 0;
		$zht = 0;
		$pht = 0;
		for($l=0; $l< 3; $l++){
			$nht+= $nvals[$l];
			$zht+= $zvals[$l];
			$pht+= $pvals[$l];
		}
		@hts = ($nht, $zht, $pht);
		$min = -(0.5*$zht+$nht);
		
		#
		# print first column of zeros
		#
		$ht = $min;
		for($l=0; $l< 3; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1], $remax*(1-$inds[$k][0]), 0);
			printf OUT ("%f %f %f\n", $inds[$k][1], $remax*(1-$inds[$k][0]), 0);
			$ht+=$hts[$l];
		}
		printf OUT ("\n");
			
		#
		# print first column of boxes
		#
		$ht = $min;
		for($l=0; $l< 3; $l++){
			printf OUT ("%f %f %f %i %i\n", $inds[$k][1], $remax*(1-$inds[$k][0]) + $ht, 2*($l-1), $inds[$k][1], $inds[$k][0]);
			printf OUT ("%f %f %f\n", $inds[$k][1], $remax*(1-$inds[$k][0]) + $ht+$hts[$l], 2*($l-1));
			$ht+=$hts[$l];
		}
		printf OUT ("\n");
		$ht = $min;
		for($l=0; $l< 3; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w/2, $remax*(1-$inds[$k][0]) + $ht, 2*($l-1));
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w/2, $remax*(1-$inds[$k][0]) + $ht+$hts[$l], 2*($l-1));
			$ht+=$hts[$l];
		}
		printf OUT ("\n");

		#
		# print second column of boxes
		#
		$ht = $min;
		for($l=0; $l< 9; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w/2, $remax*(1-$inds[$k][0]) + $ht, 2*($l%3-1));
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w/2, $remax*(1-$inds[$k][0]) + $ht+$mat[$k][$l], 2*($l%3-1));
			$ht+=$mat[$k][$l];
		}
		printf OUT ("\n");
		$ht = $min;
		for($l=0; $l< 9; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w, $remax*(1-$inds[$k][0]) + $ht, 2*($l%3-1));
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w, $remax*(1-$inds[$k][0]) + $ht+$mat[$k][$l], 2*($l%3-1));
			$ht+=$mat[$k][$l];
		}
		printf OUT ("\n");
		$ht = $min;
		for($l=0; $l< 9; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w, $remax*(1-$inds[$k][0]), 0);
			printf OUT ("%f %f %f\n", $inds[$k][1]+$w, $remax*(1-$inds[$k][0]), 0);
			$ht+=$mat[$k][$l];
		}
		printf OUT ("\n");
		$ht = $min;
		for($l=0; $l< 9; $l++){
			printf OUT ("%f %f %f\n", $inds[$k][1]+1, $remax*(1-$inds[$k][0]), 0);
			printf OUT ("%f %f %f\n", $inds[$k][1]+1, $remax*(1-$inds[$k][0]), 0);
			$ht+=$mat[$k][$l];
		}
		printf OUT ("\n");
	}
}
close(OUT);

#
# print tics values to output
#
open(OUT,">$name.logo.tics");

printf OUT ("set ylabel \"Left-hand behavior dimension index\\nBits x Direction of regulation\"\n");
printf OUT ("set xlabel \"Right-hand behavior dimension index\"\n");


printf OUT ("set ytics (");
printf OUT ("\"self\" %f", $remax);
for($k=0; $k<= $#mat; $k++){
	if( $inds[$k][0] == $inds[$k][1] ){
		printf OUT (",\"%i\" %f", $newinds[$k][1], $remax*(1-$inds[$k][0]));
	}
}
printf OUT (")\n");

$i=0;
printf OUT ("set xtics (");
for($k=0; $k<= $#mat; $k++){
	if( $inds[$k][0] == $inds[$k][1] ){
		if($i> 0 ){printf OUT (",")}
		$i++;
		printf OUT ("\"%i\" %f", $newinds[$k][1], $inds[$k][1]+0.5);
	}
}
printf OUT (")\n");



